/**********************************************************************
 *	PS2 Keyboard Library
 **********************************************************************
 *
void  kbd_init(void);			// : 
uchar kbd_getchar(void);		// : ASCII R[h擾.
uchar kbd_peekcode(void);		// : FIFOL[TOP.
uchar kbd_getcode(void);		// : FIFOL[̎o.
uchar kbd_getpress(uchar c);	// : A^C CӃL[̉ ̎擾.

 */
#if	0

// AVR

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#define	Near
#else

// PIC

#include "usb/typedefs.h"                   
#include "usb/usb.h"                         
#include "io_cfg.h"                     

#define	Near	near
#define	PROGMEM	rom
#endif

#include "config.h"
#include "ps2keyb.h"

#pragma romdata

#if	PS2KBD_USE_GETCHAR
#include "keycode.h"
#endif

#include "keyname.h"



/**********************************************************************
 *	FIFÕTCY.
 **********************************************************************
 */
#define	BUFF_SIZE 		32			//Qׂ̂.
#define	BUFF_SIZE_MASK	(BUFF_SIZE-1)

#define	PREFIX_E0		0x1
#define	PREFIX_E1		0x2			// [Pause]=E1 14,77,E1 14,F0 77
#define	PREFIX_F0		0x4

#define	KBD_SHIFT		0x1



/**********************************************************************
 *	[NGA.
 **********************************************************************
 */
#pragma udata access accessram

static	uchar bitcount;				//	0..10 : start=0,data[8],oddparity,stop=1
static	uchar inptr, outptr;		// FIFO ̎oA݃|C^.
static  uchar kbd_data;				// Holds the received	scan code
static  uchar kbd_shift;			// Holds shift , ctrl , alt status
static  uchar prefix;				// Holds last scan code prefix
static  uchar clk_last;
static	uchar watchDog;				// 0.25b Watch Dog

#define	MAX_DOG	250


#ifndef	__18F14K50
	#pragma udata
#endif

#if	PS2KBD_USE_PRESSTABLE	
static	uchar kbd_stat[64];			// 256̃L[̉ (bit table).
#endif

#pragma udata
static	uchar kbd_buffer[BUFF_SIZE];// FIFO.


#pragma code


#ifdef	__18F14K50
	// 18F14K50łLED͎gpłȂ̂ŁALED̏o͍͂sȂ.
	// ( portc bit0,bit1 PS/2ɐڑ )
	#define	LED_TEST	0
#else
	#define	LED_TEST	1
#endif


/**********************************************************************
 *	ps2kbd : 
 **********************************************************************
 */
void kbd_init(void)
{
	//ϐ̏.
	inptr  =	0;
	outptr =	0;
	bitcount  = 11;
	prefix    =	0;
	kbd_shift =	0;
	watchDog  = 0;
	clk_last  = PS2KBD_CLOCK_MASK;	// l = H

#if	PS2KBD_USE_INTERRUPT		// use PCINT8

#ifdef	_AVR_CHIP_
	//|[g̏.
	PCICR  = 1<<1;	// PCINT1 (8..14) Enable.
	PCMSK1 = 1<<0;	// PCINT8 Mask off.
#else
	// PICp.
#ifndef	__18F14K50
	INTCON2bits.RBPU =0;	// PORTB PullUp
#endif
	INTCON2bits.INTEDG0=0;	// Falling Edge荞.

	INTCONbits.INT0IE=1;	// INT0荞݋.
#endif

#endif

}

#if	PS2KBD_USE_PRESSTABLE	
/**********************************************************************
 *	ps2kbd : 128̃L[̉mask bit.
 **********************************************************************
 */
#if	0
static uchar get_press_mask(uchar c)
{
	return 1 << (c & 7);
}
static uchar get_press_idx(uchar c)
{
	return (c >> 3);
}
#else
#define	get_press_mask(c)	(1 << (c & 7) )
#define	get_press_idx(c)		  (c >> 3)
#endif

/**********************************************************************
 *	ps2kbd : 128̃L[̉ ̍XV.
 **********************************************************************
 */
static void set_press_table(uchar c,uchar pre)
{
	uchar idx  = get_press_idx(c);
	uchar mask = get_press_mask(c);
	if(pre & PREFIX_F0) {	//ꂽ,gV[PX͖.
		kbd_stat[idx] &= ~mask;
	}else{
		kbd_stat[idx] |=  mask;
	}
}

/**********************************************************************
 *	ps2kbd : 128̃L[̉ ̎擾.
 **********************************************************************
 *	  c : XLR[h.
 *	߂l  : [̏ꍇÃXLR[h̃L[́uꂽԁvɂ.
 *			  [̏ꍇÃXLR[h̃L[́uꂽԁvɂ.
 */
uchar kbd_getpress(uchar c)
{
	uchar idx  = get_press_idx(c);
	uchar mask = get_press_mask(c);
	return kbd_stat[idx] & mask;
}

#define	KBD_getpress(c)		(kbd_stat[get_press_idx(c)] & get_press_mask(c))

uchar get_shift_status(void)
{
	uchar r=0;
//	if( kbd_shift & KBD_SHIFT )	r |= 0x02;	// bit 1 LEFT SHIFT.

	if( KBD_getpress(SKEY_CTRL  )) r |= 0x01;
	if( KBD_getpress(SKEY_LSHIFT)) r |= 0x02;
	if( KBD_getpress(SKEY_ALT   )) r |= 0x04;
	if( KBD_getpress(SKEY_LWIN  )) r |= 0x08;

	if( KBD_getpress(SKEY_RCTRL )) r |= 0x10;
	if( KBD_getpress(SKEY_RSHIFT)) r |= 0x20;
	if( KBD_getpress(SKEY_RALT  )) r |= 0x40;
	if( KBD_getpress(SKEY_RWIN  )) r |= 0x80;

	return r;
}

/**********************************************************************
 *	ps2kbd : 128̃L[̉ ̃NA.
 **********************************************************************
 */
void clr_press_table(void)
{
	uchar i;
	for(i=0;i<32;i++) {
		kbd_stat[i] = 0;
	}
}
#else

#endif
/**********************************************************************
 *	ps2kbd : FIFOL[̑.
 **********************************************************************
 */
static	void put_que(uchar c)
{
	uchar p = inptr;

	if(c==0) return;			// 0 FIFOɏ܂Ȃ.

	kbd_buffer[p]=c;
	p++; p &= BUFF_SIZE_MASK;	//  p i߂Ă݂.

	if(p != outptr) {	//  p o|C^ɒǂĂȂꍇ̂,
		inptr = p;		// |C^XV.
	}

#if	0	// TEST
	mLED_1_Toggle();
#endif
}

/**********************************************************************
 *	ps2kbd : FIFOL[TOP.
 **********************************************************************
 */
uchar kbd_peekcode(void)
{
	uchar p = outptr;
	if(p == inptr) {			//L[Ɣf.
		return 0;				//͂Ȃ.
	}
	return kbd_buffer[p];		//L[TOP邾.
}

/**********************************************************************
 *	ps2kbd : FIFOL[̎o.
 **********************************************************************
 *	L[͂ȂƂ́A0Ԃ.
 *	L[͂Ƃ́AXLR[h([)1oCgԂ.
 *ӁF
 *	ASCIIR[ĥ݂擾ꍇ́Akbd_getcode()،Ă΂ɁA
 *	kbd_getchar()̂قgpB
 *	݂ČĂяoꍇAkbd_getchar()̓삪ȂB
 */
uchar kbd_getcode(void)
{
	uchar c;
	uchar p = outptr;
	if(p == inptr) {			//L[Ɣf.
		return 0;				//͂Ȃ.
	}
	c = kbd_buffer[p];			//L[o.
	p++; p &= BUFF_SIZE_MASK;	//  p i߂Ă݂.
	outptr = p;
	return c;
}

#if	PS2KBD_USE_GETCHAR
/**********************************************************************
 *	ps2kbd : getchar ASCII R[h擾.
 **********************************************************************
 *	L[͂ȂƂ́A0Ԃ.
 *	L[u[N(ꂽ)OuR[h(0xF0)gV[PX(0xE0)
 *	擾ꍇ́A prefixƂX^eBbNϐɂ̃tO
 *	cA߂l 0 ɂȂ.ĩR[h擾ɃtOQƂj
 */
uchar kbd_getchar(void)
{
	uchar pre = prefix;			//ÕR[hgR[hĂ?
	uchar c = kbd_getcode();
	uchar idx;

	if(c==0) return c;			//͖.

	// QoCgȏ̃V[PXꍇ́AOuR[hŏ:
	if(c == 0xe0) {				//gV[PX E0.
		prefix |= PREFIX_E0;
		return 0;
	}else if(c == 0xe1) {		//gV[PX E1.
		prefix |= PREFIX_E1;
		return 0;
	}else if(c == 0xf0) {		//ꂽ񂪌㑱.
		prefix |= PREFIX_F0;
		return 0;
	}

	// QoCgȏ̃V[PXȂAŌ̂PoCg:
	// ͒PL[̏ꍇɗ:

	prefix = 0;
	// [SHIFT] Ԃ̃tOXV.
	if((c == SKEY_LSHIFT)||(c == SKEY_RSHIFT)) {
		if(pre & PREFIX_F0) {
			kbd_shift &= KBD_SHIFT;
		}else{
			kbd_shift |= KBD_SHIFT;
		}
	}
	set_press_table(c,pre);

	if(pre & PREFIX_E0) {	//gV[PXSCAN CODEMSB𗧂Ăċʂ.
		c |= SKEY_EXTEND;
	}

	if(pre & PREFIX_F0) {	//ꂽ͖.
		return 0;
	}

#ifdef	_AVR_CHIP_
	idx = c*2;
	if(kbd_shift & KBD_SHIFT) idx++;	// [SHIFT]Ă?

	// SCAN CODE --> ASCII CODE
	return __LPM( &keycode_table[idx] );
#else
	idx = c;
	// PIC
	// SCAN CODE --> ASCII CODE
	return keycode_table_usb[idx];
#endif
}
#endif


#if	PS2KBD_USE_INTERRUPT

/**********************************************************************
 *	ps2kbd : 荞݃nh[.
 **********************************************************************
 */

//  PIC

//  use INT0 GbW荞 hard handler

void 	kbd_handler(void)
{
	if(INTCONbits.INT0IF) {
#if	1
		// mCY΍. CLKmCYɂāArbgꂪNĂZbg.
		if(	watchDog >= MAX_DOG) {	// Ō INT0GbW荞݂0.25bo߂.
			bitcount = 11;			// Ԍ. rbgJE^.
		}
		watchDog=0;					// watchDogZbg.


		// R falling edge肷.
											//	start and stop bits	are	ignored.	
											// Routine entered at	falling	edge
		if(bitcount	< 11 &&	bitcount > 2) {	//	Bit	3 to 10	is data. Parity	bit,
			kbd_data >>= 1;
			if(PS2KBD_PIN & PS2KBD_DATA_MASK) {
				kbd_data |= 0x80;			// Store	a '1'
			}
		}

		// rising edgȅłɂ.
		if(--bitcount == 0)	{				// All bits received
			put_que(kbd_data);
			bitcount = 11;
		}
#endif
		INTCONbits.INT0IF=0;
#if	LED_TEST
	mLED_2_Toggle();
#endif
	}
}

//	莞荞(1000Hz)p.

void 	kbd_int_handler(void)
{
	if( watchDog < MAX_DOG ) {
		watchDog++;
	}
#if	LED_TEST
	mLED_1_Toggle();
#endif

}

#else


/**********************************************************************
 *	PCINT8 TIMER1 荞݂ G~[gipj.
 **********************************************************************
 */
void 	kbd_handler(void)
{
											//	start and stop bits	are	ignored.
	if(!(PS2KBD_PIN & PS2KBD_CLOCK_MASK)) {	// Routine entered at	falling	edge
		if(bitcount	< 11 &&	bitcount > 2) {	//	Bit	3 to 10	is data. Parity	bit,
			kbd_data >>= 1;
			if(PS2KBD_PIN & PS2KBD_DATA_MASK) {
				kbd_data |= 0x80;			// Store	a '1'
			}
		}
	} else {								// Routine entered at rising edge
		if(--bitcount == 0)	{				// All bits received
			put_que(kbd_data);
			bitcount = 11;
		}
	}
}


//	use TIMER1 softhandler
void 	kbd_int_handler(void)
{
	uchar clk_edge = PS2KBD_PIN & PS2KBD_CLOCK_MASK;	//ݒl KBD_CLK

	// KBD_CLK̓dʂɕω?.(OƈقȂĂ邩)
	if(	clk_last != clk_edge ) {
		clk_last  = clk_edge;
		kbd_handler();					// PCINT8 ̃Gg[R[.
	}
}
#endif
/**********************************************************************
 *	
 **********************************************************************
 */
